package com.corelogic.infinispan;

import java.util.List;
import java.util.Properties;

import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;

import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.AvailableSettings;
import org.hibernate.cfg.Configuration;
import org.hibernate.cfg.Environment;
import org.hibernate.internal.util.config.ConfigurationHelper;
import org.hibernate.service.BootstrapServiceRegistryBuilder;
import org.hibernate.service.ServiceRegistry;
import org.hibernate.service.ServiceRegistryBuilder;

import com.corelogic.dao.KeyValueDAO;
import com.corelogic.domain.KeyValue;

@TransactionManagement(TransactionManagementType.CONTAINER)
public class JTAHibernateTransactionManager implements CLTransactionManager {
		
		private KeyValueDAO dao = null;		
		private SessionFactory sessionFactory;

		public JTAHibernateTransactionManager() {
			init();
		}
		
		@TransactionAttribute(TransactionAttributeType.NEVER)
		protected void init() {
			
			// prepare the configuration
	        Configuration configuration = 
	        		new Configuration().setProperty(AvailableSettings.USE_NEW_ID_GENERATOR_MAPPINGS,
	        										"true");        
	        configuration.setProperty(Environment.DATASOURCE, "java:jboss/datasources/mysqldbDS");           
	        
	        // fetch the properties
	        Properties properties = new Properties();
	        properties.putAll(configuration.getProperties());
	        Environment.verifyProperties(properties);
	        ConfigurationHelper.resolvePlaceHolders(properties);

	        // build the serviceregistry
	        final BootstrapServiceRegistryBuilder bootstrapbuilder = new BootstrapServiceRegistryBuilder();
	        ServiceRegistryBuilder builder = 
	        		new ServiceRegistryBuilder(bootstrapbuilder.build()).applySettings(properties);
	        ServiceRegistry serviceRegistry = builder.buildServiceRegistry();
	        
	        // Create the SessionFactory from Configuration
	        sessionFactory = configuration.configure("hibernate.cfg.xml").buildSessionFactory(serviceRegistry);        
		}
		
		public void save(Object key, Object value) {
			KeyValue transientInstance = new KeyValue();		
			transientInstance.setKey((String)key);
			transientInstance.setValue((String)value);
			Session s = openSession();
			dao.save(s, transientInstance);	
			closeSession(s);
		}
		
		public Object findById(Class clazz, Object key) {
			Session s = openSession();
			Object keyValue = dao.findById(s, clazz,(String)key);
			closeSession(s);
			return keyValue;
		}
		
		public List<Object> findAll(Class clazz) {
			Session s = openSession();
			List<Object> list = dao.findAll(s, clazz);
			closeSession(s);
			return list;
		}

		public Object delete(Class clazz, Object key) {
			Session s = openSession();
			Object keyValue = dao.findById(s, clazz,(String)key);
			if (keyValue != null) {				
				dao.delete(s, (KeyValue)keyValue);
			}
			closeSession(s);
			return keyValue;
		}
		
		public void deleteAll(Class clazz) {
			Session s = openSession();
			dao.deleteAll(s, clazz);
			closeSession(s);
		}
		
		public int size() {
			int size = 0;
			Session s = openSession();
			size = dao.findAll(s, KeyValue.class).size();
			closeSession(s);
			return size;
		}
		
		public boolean isEmpty() {
			boolean isEmpty = false;
			Session s = openSession();
			isEmpty = dao.findAll(s, KeyValue.class).isEmpty();
			closeSession(s);
			return isEmpty;
		}
		
		public boolean containsKey(Class clazz, Object key) {
			boolean isContains = false;
			Session s = openSession();
			isContains = (dao.findById(s, clazz, (String)key) != null);
			closeSession(s);
			return isContains;
		}

		public Session beginTransaction() {
			Session hibernateSession = openSession();
			hibernateSession.beginTransaction();
			return hibernateSession;
		}

		public  void commitTransaction() {
			Session hibernateSession = openSession();
			hibernateSession.getTransaction().commit();		
		}

		public  void rollbackTransaction() {
			openSession().getTransaction().rollback();		
		}
		
		public Session openSession() {
			return sessionFactory.openSession();
		}
		
		public void closeSession(Session session) {
			session.flush();
			session.close();			
		}

		public KeyValueDAO getDao() {
			return dao;
		}

		public void setDao(KeyValueDAO dao) {
			this.dao = dao;
		}

		public SessionFactory getSessionFactory() {
			return sessionFactory;
		}

		public void setSessionFactory(SessionFactory sessionFactory) {
			this.sessionFactory = sessionFactory;
	}
}